body{margin:0}@font-face{font-family:Source Sans Pro;font-weight:400;font-style:normal;src:url(../fonts/SourceSansPro-Regular.woff2) format("woff2")}@font-face{font-family:Source Sans Pro;font-weight:600;font-style:normal;src:url(../fonts/SourceSansPro-SemiBold.woff2) format("woff2")}@font-face{font-family:Source Sans Pro;font-weight:700;font-style:normal;src:url(../fonts/SourceSansPro-Bold.woff2) format("woff2")}
